@@ -250,7 +250,7 @@ public class TokenQueue {
 
     /**
      * Pulls a balanced string off the queue. E.g. if queue is "(one (two) three) four", (,) will return "one (two) three",
-     * and leave " four" on the queue. Unbalanced openers and closers can be escaped (with \). Those escapes will be left
+     * and leave " four" on the queue. Unbalanced openers and closers can quoted (with ' or ") or escaped (with \). Those escapes will be left
      * in the returned string, which is suitable for regexes (where we need to preserve the escape), but unsuitable for
      * contains text strings; use unescape for that.
      * @param open opener
@@ -262,11 +262,16 @@ public class TokenQueue {
         int end = -1;
         int depth = 0;
         char last = 0;
+        boolean inQuote = false;
 
         do {
             if (isEmpty()) break;
             Character c = consume();
             if (last == 0 || last != ESC) {
+                if ((c.equals('\'') || c.equals('"')) && c != open)
+                    inQuote = !inQuote;
+                if (inQuote)
+                    continue;
                 if (c.equals(open)) {
                     depth++;
                     if (start == -1)
