@@ -17,12 +17,15 @@ import org.junit.Test;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
+import java.net.ConnectException;
+import java.net.InetSocketAddress;
 import java.net.MalformedURLException;
+import java.net.Proxy;
 import java.net.URL;
 import java.util.Map;
 
-import static org.hamcrest.core.Is.is;
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
 
 /**
  Tests the URL connection. Not enabled by default, so tests don't require network connection.
@@ -33,6 +36,7 @@ public class UrlConnectTest {
     private static final String WEBSITE_WITH_INVALID_CERTIFICATE = "https://certs.cac.washington.edu/CAtest/";
     private static final String WEBSITE_WITH_SNI = "https://jsoup.org/";
     private static String echoURL = "http://direct.infohound.net/tools/q.pl";
+    private static String browserUa = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/49.0.2623.112 Safari/537.36";
 
     @Test
     public void fetchURl() throws IOException {
@@ -109,6 +113,32 @@ public class UrlConnectTest {
         assertEquals("Jsoup, Jonathan", ihVal("uname", doc));
     }
 
+    @Test
+    public void sendsRequestBody() throws IOException {
+        final String body = "{key:value}";
+        Document doc = Jsoup.connect(echoURL)
+            .requestBody(body)
+            .header("Content-Type", "text/plain")
+            .userAgent(browserUa)
+            .post();
+        assertEquals("POST", ihVal("REQUEST_METHOD", doc));
+        assertEquals(body, ihVal("keywords", doc));
+    }
+
+    @Test
+    public void sendsRequestBodyWithUrlParams() throws IOException {
+        final String body = "{key:value}";
+        Document doc = Jsoup.connect(echoURL)
+            .requestBody(body)
+            .data("uname", "Jsoup", "uname", "Jonathan", "百", "度一下")
+            .header("Content-Type", "text/plain") // todo - if user sets content-type, we should append postcharset
+            .userAgent(browserUa)
+            .post();
+        assertEquals("POST", ihVal("REQUEST_METHOD", doc));
+        assertEquals("uname=Jsoup&uname=Jonathan&%E7%99%BE=%E5%BA%A6%E4%B8%80%E4%B8%8B", ihVal("QUERY_STRING", doc));
+        assertEquals(body, ihVal("POSTDATA", doc));
+    }
+
     @Test
     public void doesGet() throws IOException {
         Connection con = Jsoup.connect(echoURL + "?what=the")
@@ -282,7 +312,7 @@ public class UrlConnectTest {
         Connection.Response res = con.execute();
         assertEquals("asdfg123", res.cookie("token")); // confirms that cookies set on 1st hit are presented in final result
         Document doc = res.parse();
-        assertEquals("uid=jhy; token=asdfg123", ihVal("HTTP_COOKIE", doc)); // confirms that redirected hit saw cookie
+        assertEquals("token=asdfg123; uid=jhy", ihVal("HTTP_COOKIE", doc)); // confirms that redirected hit saw cookie
     }
 
     @Test
@@ -309,7 +339,7 @@ public class UrlConnectTest {
 
         // send those cookies into the echo URL by map:
         Document doc = Jsoup.connect(echoURL).cookies(cookies).get();
-        assertEquals("uid=jhy; token=asdfg123", ihVal("HTTP_COOKIE", doc));
+        assertEquals("token=asdfg123; uid=jhy", ihVal("HTTP_COOKIE", doc));
     }
 
     @Test
@@ -389,7 +419,7 @@ public class UrlConnectTest {
     public void testSNIPass() throws Exception {
         String url = WEBSITE_WITH_SNI;
         Connection.Response defaultRes = Jsoup.connect(url).validateTLSCertificates(false).execute();
-        assertThat(defaultRes.statusCode(), is(200));
+        assertEquals(defaultRes.statusCode(), 200);
     }
 
     /**
@@ -403,7 +433,7 @@ public class UrlConnectTest {
     public void testUnsafePass() throws Exception {
         String url = WEBSITE_WITH_INVALID_CERTIFICATE;
         Connection.Response defaultRes = Jsoup.connect(url).validateTLSCertificates(false).execute();
-        assertThat(defaultRes.statusCode(), is(200));
+        assertEquals(defaultRes.statusCode(), 200);
     }
 
     @Test
@@ -464,14 +494,10 @@ public class UrlConnectTest {
 
         File uploadFile = ParseTest.getFile("/htmltests/google-ipod.html");
         FileInputStream stream = new FileInputStream(uploadFile);
-
-        // todo: need to add a better way to get an existing data field
-        for (Connection.KeyVal keyVal : post.request().data()) {
-            if (keyVal.key().equals("_file")) {
-                keyVal.value("check.html");
-                keyVal.inputStream(stream);
-            }
-        }
+        
+        Connection.KeyVal fileData = post.data("_file");
+        fileData.value("check.html");
+        fileData.inputStream(stream);
 
         Connection.Response res;
         try {
@@ -491,11 +517,13 @@ public class UrlConnectTest {
     public void postJpeg() throws IOException {
         File thumb = ParseTest.getFile("/htmltests/thumb.jpg");
         Document result = Jsoup
-                .connect("http://regex.info/exif.cgi")
-                .data("f", thumb.getName(), new FileInputStream(thumb))
-                .post();
+            .connect("http://regex.info/exif.cgi")
+            .data("f", thumb.getName(), new FileInputStream(thumb))
+            .userAgent(browserUa)
+            .post();
 
         assertEquals("Baseline DCT, Huffman coding", result.select("td:contains(Process) + td").text());
+        assertEquals("1052 bytes 30 × 30", result.select("td:contains(Size) + td").text());
     }
 
     @Test
@@ -538,4 +566,81 @@ public class UrlConnectTest {
         assertEquals("<html> <head></head> <body> <xml> <link>one <table> Two </table> </xml> </body> </html>", StringUtil.normaliseWhitespace(doc.outerHtml()));
     }
 
+    @Test
+    public void combinesSameHeadersWithComma() throws IOException {
+        // http://www.w3.org/Protocols/rfc2616/rfc2616-sec4.html#sec4.2
+        String url = "http://direct.infohound.net/tools/q.pl";
+        Connection con = Jsoup.connect(url);
+        con.get();
+
+        assertEquals("text/html", con.response().header("Content-Type"));
+        assertEquals("no-cache, no-store", con.response().header("Cache-Control"));
+    }
+
+    @Test
+    public void sendHeadRequest() throws IOException {
+        String url = "http://direct.infohound.net/tools/parse-xml.xml";
+        Connection con = Jsoup.connect(url).method(Connection.Method.HEAD);
+        final Connection.Response response = con.execute();
+        assertEquals("text/xml", response.header("Content-Type"));
+        assertEquals("", response.body()); // head ought to have no body
+        Document doc = response.parse();
+        assertEquals("", doc.text());
+    }
+
+
+    /*
+     Proxy tests. Assumes local proxy running on 8888, without system propery set (so that specifying it is required).
+     */
+
+    @Test
+    public void fetchViaHttpProxy() throws IOException {
+        String url = "http://jsoup.org";
+        Proxy proxy = new Proxy(Proxy.Type.HTTP, InetSocketAddress.createUnresolved("localhost", 8888));
+        Document doc = Jsoup.connect(url).proxy(proxy).get();
+        assertTrue(doc.title().contains("jsoup"));
+    }
+
+    @Test
+    public void fetchViaHttpProxySetByArgument() throws IOException {
+        String url = "http://jsoup.org";
+        Document doc = Jsoup.connect(url).proxy("localhost", 8888).get();
+        assertTrue(doc.title().contains("jsoup"));
+    }
+
+    @Test
+    public void invalidProxyFails() throws IOException {
+        boolean caught = false;
+        String url = "http://jsoup.org";
+        try {
+            Document doc = Jsoup.connect(url).proxy("localhost", 8889).get();
+        } catch (IOException e) {
+            caught = e instanceof ConnectException;
+        }
+        assertTrue(caught);
+    }
+
+    @Test
+    public void proxyGetAndSet() throws IOException {
+        String url = "http://jsoup.org";
+        Proxy proxy = new Proxy(Proxy.Type.HTTP, InetSocketAddress.createUnresolved("localhost", 8889)); // invalid
+        final Connection con = Jsoup.connect(url).proxy(proxy);
+
+        assert con.request().proxy() == proxy;
+        con.request().proxy(null); // disable
+        Document doc = con.get();
+        assertTrue(doc.title().contains("jsoup")); // would fail if actually went via proxy
+    }
+
+    @Test
+    public void throwsIfRequestBodyForGet() throws IOException {
+        boolean caught = false;
+        String url = "http://jsoup.org";
+        try {
+            Document doc = Jsoup.connect(url).requestBody("fail").get();
+        } catch (IllegalArgumentException e) {
+            caught = true;
+        }
+        assertTrue(caught);
+    }
 }
